#!/usr/bin/guile -s
!#
;;     This file is part of cdloop
;;     Copyright (C) 2001-2003 Claus Brunzema <mail@cbrunzema.de>
;; 
;;     This program is free software; you can redistribute it and/or modify
;;     it under the terms of the GNU General Public License as published by
;;     the Free Software Foundation; either version 2 of the License, or
;;     (at your option) any later version.
;; 
;;     This program is distributed in the hope that it will be useful,
;;     but WITHOUT ANY WARRANTY; without even the implied warranty of
;;     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;     GNU General Public License for more details.
;; 
;;     You should have received a copy of the GNU General Public License
;;     along with this program; if not, write to the Free Software
;;     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;; 

(cond ((< (length (command-line)) 2)
       (display "Usage:\n")
       (display (car (command-line)))
       (display " <name>\n")
       (display "reads <name>.def and makes built_<name>.hh with enum-declaration and\n")
       (display "built_<name>_to_symbol.hh/built_<name>_to_symbol.cc\n with conversion function.\n")
       (quit)))

(let ((name (cadr (command-line))))
  (let ((headerfile (open-output-file (string-append "built_"
                                                     name
                                                     "_to_symbol.hh"))))
  (write-line "// -*- C++ -*-" headerfile)
  (write-line (string-append "#ifndef BUILT_"
                             (string-upcase name)
                             "_TO_SYMBOL_HH_INCLUDED")
              headerfile)
  (write-line (string-append "#define BUILT_"
                             (string-upcase name)
                             "_TO_SYMBOL_HH_INCLUDED")
              headerfile)
  (write-line "#include <guile/gh.h>" headerfile)
  (write-line (string-append "#include \"built_" name ".hh\"") headerfile)
  (write-line (string-append "extern SCM " name "_to_symbol(" name ");") headerfile)
  (write-line "#endif" headerfile)
  (close headerfile))

  (let((headerfile (open-output-file (string-append "built_" name ".hh")))
       (ccfile (open-output-file (string-append "built_"
                                                name
                                                "_to_symbol.cc"))))
    (write-line "// -*- C++ -*-" headerfile)
    (write-line (string-append
                 "#ifndef BUILT_" (string-upcase name) "_HH_INCLUDED")
                headerfile)
    (write-line (string-append
                 "#define BUILT_" (string-upcase name) "_HH_INCLUDED")
                headerfile)
    (write-line (string-append "\nenum " name " {") headerfile)
    
    (write-line "// -*- C++ -*-" ccfile)
    (write-line (string-append "#include \"built_" name "_to_symbol.hh\"\n") ccfile)
    (write-line (string-append "SCM " name "_to_symbol(" name " _in) {") ccfile)
    (write-line " switch(_in) {" ccfile)
    
    (call-with-input-file (string-append name ".def")
      (lambda (in)
        (let ((count 0))
          (while (not (eof-object? (peek-char in)))
                 (let ((sym (read-line in 'trim)))
                   (if (and (> (string-length sym) 0)
                            (not (char=? (string-ref sym 0) #\;)))
                       (begin
                         (if (> count 0)
                             (write-line "," headerfile))
                         (display (string-append sym "=" (number->string count)) headerfile)
                   
                         (write-line (string-append
                                      "  case " sym ": return gh_symbol2scm(\"" sym "\");")
                                     ccfile)
                         (set! count (+ 1 count)))))))))
    
    (write-line "\n};\n" headerfile)
    (write-line "#endif" headerfile)
    
    (write-line " }" ccfile)
    (write-line " return gh_symbol2scm(\"STATUS_ERROR\");" ccfile)
    (write-line "}" ccfile)
    
    (close headerfile)
    (close ccfile)))
  