#!/usr/bin/guile -s
!#
;;     This file is part of cdloop
;;     Copyright (C) 2001-2003 Claus Brunzema <mail@cbrunzema.de>
;; 
;;     This program is free software; you can redistribute it and/or modify
;;     it under the terms of the GNU General Public License as published by
;;     the Free Software Foundation; either version 2 of the License, or
;;     (at your option) any later version.
;; 
;;     This program is distributed in the hope that it will be useful,
;;     but WITHOUT ANY WARRANTY; without even the implied warranty of
;;     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;     GNU General Public License for more details.
;; 
;;     You should have received a copy of the GNU General Public License
;;     along with this program; if not, write to the Free Software
;;     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;; 

(define string-width 80)

(cond ((< (length (command-line)) 3)
       (display "Usage:\n")
       (display (car (command-line)))
       (display " <varname> <filebasename> [<inputfilename>]\n")
       (quit)))

(define varname (list-ref (command-line) 1))
(define filebasename (list-ref (command-line) 2))
(define inputfilename #f)
(if (> (length (command-line)) 3)
    (set! inputfilename (list-ref (command-line) 3)))

(with-output-to-file (string-append filebasename ".h")
  (lambda ()
    (let ((macroname (string-append (string-upcase filebasename)
                                    "_H_INCLUDED")))
      (display "#ifndef ")
      (display macroname)
      (display "\n#define ")
      (display macroname)
      (display "

#ifdef __cplusplus
extern \"C\" {
#endif /* __cplusplus */

")
      (display "extern char* ")
      (display varname)
      (display ";\n")
      (display "
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif")
      (newline))))

(define (make-string-lines input-port)
  (let ((count 0))
    (while (not (eof-object? (peek-char input-port)))
           (if (= count 0) (display "\""))
           (display "\\x")
           (display (number->string (char->integer (read-char input-port)) 16))
           (if (= count (- string-width 1)) (display "\"\n"))
           (set! count (modulo (+ 1 count) string-width)))
    (if (> count 0) (display "\""))))

(with-output-to-file (string-append filebasename ".c")
  (lambda ()
    (display "#include \"")
    (display (string-append filebasename ".h"))
    (display "\"

char* ")
    (display varname)
    (display "=\n")
    (if inputfilename
        (call-with-input-file inputfilename make-string-lines)
        (make-string-lines (current-input-port)))
    (display ";")
    (newline)))
